"""
Tethne is a package for analyzing citation data from the Web of Science.
Modules within Tethne can generate a variety of networks, such as
bibliographic coupling, citation, author-paper, and co-author networks,
using networkx.

.. autosummary::

   tethne.analyze
   tethne.builders
   tethne.data
   tethne.matrices
   tethne.networks
   tethne.readers
   tethne.utilities
   tethne.writers
   
"""
