"""
The :mod:`tethne.analyze` sub-package provides additional analysis methods not
provided by NetworkX, as well as methods for using NetworkX algorithms on
an entire :class:`.GraphCollection`\.

.. autosummary::

   collection
   graph

"""

import tethne.analyze.collection
import tethne.analyze.graph
